const Dir = Object.freeze({ UP: 0, DOWN: 1, LEFT: 2, RIGHT: 3, CENTER: 4 })

class Letter {
	constructor(name, dirs, code) {
		this.name = name
		this.dirs = dirs
		this.code = code
	}
}

let textEdit = null

let SPEED = 0
let PAUSE_LENGHT = 0
let INDICATORS_SIZE_BIG = 12
let INDICATORS_SIZE_SMALL = 5
let INDICATORS_OFFSET = 8

let width = window.innerWidth - INDICATORS_SIZE_BIG
let height = window.innerHeight - INDICATORS_SIZE_BIG * 2
let x = width / 2
let y = height / 2

const ELLIPCE_SIZE = 44
//    regex
const ALL_SYMBOLS = 'Ũ'

const codes = [
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
	{ key: '', value: '' },
]

let alphabet = []
let letters = []
let array = []
let ctx = null

function setup() {
	createCanvas(width, height)
	ctx = canvas.getContext('2d')
	fillAlphabet()
	textEdit = document.getElementById('myedit')
	SPEED = document.getElementById('speedSlider').value
	PAUSE_LENGHT = document.getElementById('pauseSlider').value
	document.getElementById('stopButton').disabled = true
	textEdit.value = ALL_SYMBOLS
}

let letter = 0
let step = 0

let newX = 0
let newY = 0

let isFirst = true
let isCanStart = false
let isDiagonal = true
let isIndicators = true
let isEnd = false

function draw() {
	background(100)
	if (isEnd) {
		ctx.fillStyle = '#FFFF00'
		ctx.fillText('CONGRATULATIONS!', x - 110, height / 2 - 200)
		ctx.fillStyle = '#00FF00'
		ctx.fillText('    !', x - 230, height / 2 - 150)
	}
	if (isCanStart) {
		if (isFirst) {
			noLoop()
			setTimeout(loop, PAUSE_LENGHT)
			ctx.fillStyle = '#000000'
			ctx.font = '15px Arial'
			ctx.fillText(
				letters[letter].code,
				x - ELLIPCE_SIZE / 2 + 6,
				y + ELLIPCE_SIZE + INDICATORS_SIZE_BIG
			)
			isFirst = false
		}
		newX = array[letter][step][0]
		newY = array[letter][step][1]

		if (step + 1 == array[letter].length) {
			ctx.fillStyle = '#E8FF98'
		} else {
			ctx.fillStyle = 'white'
		}
		ellipse(x, y, ELLIPCE_SIZE, ELLIPCE_SIZE)
		ctx.font = '32px Arial'
		ctx.fillStyle = 'black'
		ctx.fillText(letters[letter].name, x - 12, y + 12)

		let isReady = false
		let dir = letters[letter].dirs[step]
		let prevDir = -1
		if (step - 1 >= 0) {
			prevDir = letters[letter].dirs[step - 1]
		}
		let nextDir = -1
		if (step + 1 < letters[letter].dirs.length) {
			nextDir = letters[letter].dirs[step + 1]
		}

		let dx = newX - x
		let dy = newY - y

		if (dir == Dir.LEFT && x <= newX) {
			isReady = true
		} else if (dir == Dir.RIGHT && x >= newX) {
			isReady = true
		} else if (dir == Dir.UP && y <= newY) {
			isReady = true
		} else if (dir == Dir.DOWN && y >= newY) {
			isReady = true
		} else if (dir == Dir.CENTER && Math.sqrt(dx * dx + dy * dy) < SPEED / 2) {
			isReady = true
		}

		// actual movement dir
		drawDir(dir, '#FFDE6F', 0)

		// next movement dir
		drawDir(nextDir, '#94DD9C', 1)

		if (isReady && array[letter].length > step) {
			step++
		}

		if (step + 1 > array[letter].length && array.length > letter + 1) {
			ctx.fillStyle = 'white'
			ellipse(x, y, ELLIPCE_SIZE, ELLIPCE_SIZE)

			letter++
			step = 0

			ctx.fillStyle = '#000000'
			ctx.fillText(letters[letter].name, x - 12, y + 12)

			//   
			ctx.font = '15px Arial'
			ctx.fillText(
				letters[letter].code,
				x - ELLIPCE_SIZE / 2 + 6,
				y + ELLIPCE_SIZE + INDICATORS_SIZE_BIG
			)

			// next movement dir
			nextDir = letters[letter].dirs[0]
			drawDir(nextDir, '#94DD9C', 1)

			noLoop()
			setTimeout(loop, PAUSE_LENGHT)
		} else if (step == array[letter].length && array.length == letter + 1) {
			isEnd = true
			onStop()
		}

		let angle = atan2(dy, dx)
		let xVelocity = SPEED * cos(angle)
		let yVelocity = SPEED * sin(angle)

		//  ,    
		if (
			Math.sqrt(dx * dx + dy * dy) <
			Math.sqrt(height * height + width * width) / SPEED / 10
		) {
			xVelocity *= (0.1+SPEED/10000)
			yVelocity *= (0.1+SPEED/10000)
		}

		x += xVelocity
		y += yVelocity
	}
}

function drawDir(direction, color, reduceSizeBy) {
	if (!isIndicators) {
		return
	}

	ctx.fillStyle = color
	if (direction == Dir.LEFT) {
		ellipse(
			x - ELLIPCE_SIZE + INDICATORS_SIZE_SMALL + INDICATORS_OFFSET,
			y,
			INDICATORS_SIZE_BIG - reduceSizeBy,
			INDICATORS_SIZE_SMALL - reduceSizeBy
		)
	} else if (direction == Dir.RIGHT) {
		ellipse(
			x + ELLIPCE_SIZE - INDICATORS_SIZE_SMALL - INDICATORS_OFFSET,
			y,
			INDICATORS_SIZE_BIG - reduceSizeBy,
			INDICATORS_SIZE_SMALL - reduceSizeBy
		)
	} else if (direction == Dir.UP) {
		ellipse(
			x,
			y - ELLIPCE_SIZE + INDICATORS_SIZE_SMALL + INDICATORS_OFFSET,
			INDICATORS_SIZE_SMALL - reduceSizeBy,
			INDICATORS_SIZE_BIG - reduceSizeBy
		)
	} else if (direction == Dir.DOWN) {
		ellipse(
			x,
			y + ELLIPCE_SIZE - INDICATORS_SIZE_SMALL - INDICATORS_OFFSET,
			INDICATORS_SIZE_SMALL - reduceSizeBy,
			INDICATORS_SIZE_BIG - reduceSizeBy
		)
	}
}

function fillArrayFrom(symbols) {
	// copy of alphabet
	let tempAlphabet = []
	for (let i = 0; i < alphabet.length; i++) {
		let tempDirs = []
		for (let j = 0; j < alphabet[i].dirs.length; j++) {
			tempDirs.push(alphabet[i].dirs[j])
		}
		tempAlphabet[i] = new Letter(alphabet[i].name, tempDirs, alphabet[i].code)
	}

	for (let i = 0; i < symbols.length; i++) {
		let index = findInLetters(symbols[i])
		if (index != -1) {
			letters.push(tempAlphabet[index])
		}
	}
	fillArray()
}

function findInLetters(symbol) {
	let index = -1
	for (let i = 0; i < alphabet.length; i++) {
		if (alphabet[i].name == symbol.toUpperCase()) {
			index = i
		}
	}
	return index
}

function fillArray() {
	console.table(letters)
	for (let i = 0; i < letters.length; i++) {
		let inner = []
		for (let j = 0; j < letters[i].dirs.length; j++) {
			if (!isDiagonal) {
				if (
					(letters[i].dirs[j] == Dir.LEFT || letters[i].dirs[j] == Dir.RIGHT) &&
					(letters[i].dirs[j - 1] == Dir.UP ||
						letters[i].dirs[j - 1] == Dir.DOWN)
				) {
					inner.push([width / 2, height / 2])
					letters[i].dirs.splice(j, 0, Dir.CENTER)
				} else if (
					(letters[i].dirs[j] == Dir.UP || letters[i].dirs[j] == Dir.DOWN) &&
					(letters[i].dirs[j - 1] == Dir.LEFT ||
						letters[i].dirs[j - 1] == Dir.RIGHT)
				) {
					inner.push([width / 2, height / 2])
					letters[i].dirs.splice(j, 0, Dir.CENTER)
				}
			}

			if (letters[i].dirs[j] == Dir.UP) {
				inner.push([width / 2, ELLIPCE_SIZE])
			} else if (letters[i].dirs[j] == Dir.DOWN) {
				inner.push([width / 2, height - ELLIPCE_SIZE])
			} else if (letters[i].dirs[j] == Dir.LEFT) {
				inner.push([ELLIPCE_SIZE, height / 2])
			} else if (letters[i].dirs[j] == Dir.RIGHT) {
				inner.push([width - ELLIPCE_SIZE, height / 2])
			}
		}
		//      
		inner.push([width / 2, height / 2])
		letters[i].dirs.push(Dir.CENTER)
		//  
		array.push(inner)
	}
}

//  ,     ''
function fillAlphabet() {
	for (let i = 0; i < codes.length; i++) {
		let tempName = codes[i].key
		let tempCode = codes[i].value
		let tempDirs = calcAndGetDirsFrom(tempCode)
		alphabet[i] = new Letter(tempName, tempDirs, tempCode)
	}
}

function calcAndGetDirsFrom(element) {
	let tempDirs = []
	for (let i = 0; i < element.length; i++) {
		if (element[i] == '') {
			tempDirs.push(Dir.LEFT)
			tempDirs.push(Dir.RIGHT)
		} else if (element[i] == '') {
			tempDirs.push(Dir.RIGHT)
			tempDirs.push(Dir.LEFT)
		} else if (element[i] == '') {
			tempDirs.push(Dir.UP)
			tempDirs.push(Dir.DOWN)
		} else if (element[i] == '') {
			tempDirs.push(Dir.DOWN)
			tempDirs.push(Dir.UP)
		}
	}
	return tempDirs
}

function onBegin() {
	if (
		!/^[Ũ]+$/.test(
			textEdit.value
		)
	) {
		alert(
			'   .  , !\n    ""  "" (('
		)
		return
	}
	document.getElementById('beginButton').disabled = true
	document.getElementById('stopButton').disabled = false

	let symbols = textEdit.value
	let output = symbols.split('')

	fillArrayFrom(output)
	isCanStart = true
	isEnd = false
}

function onStop() {
	document.getElementById('stopButton').disabled = true
	document.getElementById('beginButton').disabled = false
	isCanStart = false
	isFirst = true
	letter = 0
	step = 0
	x = width / 2
	y = height / 2
	for (let i = 0; i < letters.length; i++) {
		letters[i].dirs = []
	}
	letters = []

	array = []
}

function resumeToAlphabet() {
	textEdit.value = ALL_SYMBOLS
}

function onSpeedChange() {
	let value = document.getElementById('speedSlider').value
	SPEED = value
}

function onPauseChange() {
	let value = document.getElementById('pauseSlider').value
	PAUSE_LENGHT = value
}

function onDiagonalClick() {
	let checkbox = document.getElementById('diagonalCheckbox')
	isDiagonal = checkbox.checked
}

function onIndicatorsClick() {
	let checkbox = document.getElementById('indicatorsCheckbox')
	isIndicators = checkbox.checked
}

function help() {
	let message =
		'!         ,     . \n ,  ,         "  ",     "". \n     ,      ,     .  ,       ,  ( = /,  = /,  = /   = /). \n  ,     ,   - .      -   . \n    .\n        . \n"" -  . \n     . \n   ,     . \n     ( ""  ""). \n        .\n     ,       .'
	alert(message)
}
